import { hasNotValue, hasValue, recordReduceMerge, sanitizeValue } from '@cocokits/common-utils';
// eslint-disable-next-line max-lines-per-function
export function validateUiComponentProps({ componentName, componentProps, uiComponentsConfig }) {
    if (!uiComponentsConfig) {
        throw new Error(`'UIComponentConfig' has not provided in the root of application`);
    }
    const componentConfig = uiComponentsConfig[componentName];
    // 1- Check if the theme supports the specified component
    if (!componentConfig) {
        const validComponents = Object.keys(uiComponentsConfig).join(', ');
        throw new Error(`This theme does not support the '${componentName}' component. Please select a different theme that supports this component or choose from available components: ${validComponents}`);
    }
    // Validate properties provided for the component against theme configuration
    // TODO: Duplicate code. merge it with `Object.entries` at bottom
    Object.entries(componentConfig).filter((entry)=>entry[0] !== 'additional').forEach((entry)=>{
        const propName = entry[0];
        const propConfig = entry[1];
        const componentPropValue = sanitizeValue(componentProps[propName]);
        if (componentPropValue !== null && componentPropValue !== undefined && typeof componentPropValue !== typeof (propConfig == null ? void 0 : propConfig.values[0])) {
            throw new Error(`The '${componentName}' component does not support the '${propName}' property as '${typeof componentPropValue}'. Valid type is: ${typeof (propConfig == null ? void 0 : propConfig.values[0])}`);
        }
        // 2- Check unsupported properties
        if (propConfig === null) {
            if (hasValue(componentProps[propName])) {
                const validProps = Object.values(componentConfig).filter((config)=>!!config).join(', ');
                throw new Error(`The '${componentName}' component does not support the '${propName}' property in this theme. Valid properties include: ${validProps}`);
            }
            return;
        }
        // 3- Check required properties are provided
        if (propConfig.require && hasNotValue(componentPropValue)) {
            throw new Error(`The '${propName}' property is required for the '${componentName}' component in this theme but was not provided.`);
        }
        // 4- Check for valid values if the property is configured to accept specific values
        if (hasValue(componentPropValue) && !propConfig.values.includes(componentPropValue)) {
            throw new Error(`'${componentPropValue}' is an invalid value for '${propName}' in '${componentName}'. Accepted values in this theme are: ${propConfig.values.join(', ')}`);
        }
    });
    // Validate properties provided for the component against theme configuration
    // TODO: Duplicate code. merge it with `Object.entries` at top
    if (componentConfig.additional) {
        Object.entries(componentConfig.additional).forEach((entry)=>{
            const propName = entry[0];
            const propConfig = entry[1];
            const componentPropValue = sanitizeValue(componentProps[propName]);
            if (componentPropValue !== null && componentPropValue !== undefined && typeof componentPropValue !== typeof (propConfig == null ? void 0 : propConfig.values[0])) {
                throw new Error(`The '${componentName}' component does not support the '${propName}' property as '${typeof componentPropValue}'. Valid type is: ${typeof (propConfig == null ? void 0 : propConfig.values[0])}`);
            }
            // 2- Check unsupported properties
            if (propConfig === null) {
                if (hasValue(componentProps[propName])) {
                    const validProps = Object.values(componentConfig).filter((config)=>!!config).join(', ');
                    throw new Error(`The '${componentName}' component does not support the '${propName}' property in this theme. Valid properties include: ${validProps}`);
                }
                return;
            }
            // 3- Check required properties are provided
            if (propConfig.require && hasNotValue(componentPropValue)) {
                throw new Error(`The '${propName}' property is required for the '${componentName}' component in this theme but was not provided.`);
            }
            // 4- Check for valid values if the property is configured to accept specific values
            if (hasValue(componentPropValue) && !propConfig.values.includes(componentPropValue)) {
                throw new Error(`'${componentPropValue}' is an invalid value for '${propName}' in '${componentName}'. Accepted values in this theme are: ${propConfig.values.join(', ')}`);
            }
        });
    }
}
export function getComponentPropsWithDefault({ componentName, componentProps, uiComponentsConfig }) {
    var _uiComponentsConfig_componentName_type, _uiComponentsConfig_componentName_color, _uiComponentsConfig_componentName_size;
    var _uiComponentsConfig_componentName_additional;
    const additional = recordReduceMerge((_uiComponentsConfig_componentName_additional = uiComponentsConfig == null ? void 0 : uiComponentsConfig[componentName].additional) != null ? _uiComponentsConfig_componentName_additional : {}, (value, key)=>{
        var _componentProps_additional;
        return {
            [key]: valueOrDefault((_componentProps_additional = componentProps.additional) == null ? void 0 : _componentProps_additional[key], value == null ? void 0 : value.default)
        };
    });
    return {
        type: valueOrDefault(componentProps.type, uiComponentsConfig == null ? void 0 : (_uiComponentsConfig_componentName_type = uiComponentsConfig[componentName].type) == null ? void 0 : _uiComponentsConfig_componentName_type.default, {
            acceptNull: false
        }),
        color: valueOrDefault(componentProps.color, uiComponentsConfig == null ? void 0 : (_uiComponentsConfig_componentName_color = uiComponentsConfig[componentName].color) == null ? void 0 : _uiComponentsConfig_componentName_color.default),
        size: valueOrDefault(componentProps.size, uiComponentsConfig == null ? void 0 : (_uiComponentsConfig_componentName_size = uiComponentsConfig[componentName].size) == null ? void 0 : _uiComponentsConfig_componentName_size.default),
        additional
    };
}
function valueOrDefault(value, defaultValue, { acceptNull = true } = {}) {
    if (acceptNull && value === null) {
        return null;
    }
    return hasValue(value) ? value : defaultValue;
}
